# IQ game
# written by Aleksander Sadikov, November 2007
# for AI practical classes (search topics)

from random import randint

class Position:
    def __init__(self, hole = -1):
        # 0 is empty space (hole), 1 is peg, -1 is out of bounds
        self.State = [[-1,1][x / 5 >= x % 5] for x in range(25)]
        if hole < 0 or hole / 5 < hole % 5 or hole >= 25:
            r = randint(0,4)
            c = randint(0,r)
            self.State[r * 5 + c] = 0
        else:
            self.State[hole] = 0

    def PrintPosition(self):
        for r in range(5):
            for c in range(r+1):
                print self.State[r * 5 + c],
            print

    def Solved(self):
        if self.GenerateMoves() == [] and (sum(self.State) == -9  or sum(self.State) >= -2):
            return True
        return False

    def ExecuteSequence(self, sequence):
        for move in sequence:
            self.Move(move)

    def UndoSequence(self, sequence):
        for move in reversed(sequence):
            self.UndoMove(move)
    
    def Move(self, (move, cost)):
        r, c = divmod(move[0], 5)
        dr, dc = int(move[1].real), int(move[1].imag)
        self.State[r * 5 + c] = 0
        self.State[(r+dr) * 5 + c + dc] = 0
        self.State[(r+2*dr) * 5 + c + 2*dc] = 1

    def UndoMove(self, (move, cost)):
        r, c = divmod(move[0], 5)
        dr, dc = int(move[1].real), int(move[1].imag)
        self.State[r * 5 + c] = 1
        self.State[(r+dr) * 5 + c + dc] = 1
        self.State[(r+2*dr) * 5 + c + 2*dc] = 0
        
    def GenerateMoves(self):
        # moves possible in 8 directions: up (U, -1j), down (D, 1j), left (L, -1), right (R, 1), and 4 diagonals (UL, UR, DL, DR)
        # costs of all moves are 1
        moves = []
        for x in range(25):
            if self.State[x] == 1:
                r,c = divmod(x,5)
                if r > 1 and self.State[x-5] == 1 and self.State[x-10] == 0:
                    moves.append(((x,-1+0j), 1))
                if r < 3 and self.State[x+5] == 1 and self.State[x+10] == 0:
                    moves.append(((x,1+0j), 1))
                if c > 1 and self.State[x-1] == 1 and self.State[x-2] == 0:
                    moves.append(((x,-1j), 1))
                if c < 3 and self.State[x+1] == 1 and self.State[x+2] == 0:
                    moves.append(((x,1j), 1))
                if r > 1 and c > 1 and self.State[x-6] == 1 and self.State[x-12] == 0:
                    moves.append(((x,-1-1j), 1))
                if r > 1 and c < 3 and self.State[x-4] == 1 and self.State[x-8] == 0:
                    moves.append(((x,-1+1j), 1))
                if r < 3 and c > 1 and self.State[x+4] == 1 and self.State[x+8] == 0:
                    moves.append(((x,1-1j), 1))
                if r < 3 and c < 3 and self.State[x+6] == 1 and self.State[x+12] == 0:
                    moves.append(((x,1+1j), 1))
        return moves
    





                